
                OBJFILE "AllocNewStack.o"

;****** AllocNewStack.asm/AllocNewStack *************************************
;
;
;    NAME
;       AllocNewStack -- allocates a new stack
;
;    SYNOPSIS
;       success = AllocNewStack( StackDataBuffer, Stacksize)
;       D0                       A0               D0
;
;       LONG    success;
;       ULONG   StackDataBuffer[3];
;       ULONG   Stacksize;
;
;    FUNCTION
;       Allocates a block of memory and initializes it as new systemstack
;       for the process. If OS 2.04 is available, exec's StackSwap() is
;       used.
;
;       The datas of the old stack are saved in the StackDataBuffer, for
;       later restoration.
;
;    INPUTS
;       NewStackBuffer = 3-longword-buffer to hold the data of the old
;               stack.
;
;       stacksize = size of the new stack in byte
;
;    RESULT
;       success (-1) or failure (0)
;
;    ATTENTION
;       You have to restore the original system stack before terminating
;       the process. Use RestoreStack().
;
;       If you're calling this routine from C language, make shure that
;       the StackDataBuffer is not allocated on the stack itself, or
;       your program will crash. You can avoid this by declaring the
;       buffer as static or gobal.
;
;
;    EXAMPLE
;       void main(void) {
;               static ULONG NewStack[3];
;
;               AllocNewStack(NewStack, 10240L);
;               .
;               .       /* do some stuff */
;               .
;               RestoreStack(NewStack);
;       }
;
;
;    SEE ALSO
;       RestoreStack()
;
;****************************************************************************

                XDEF    _AllocNewStack

_AllocNewStack:
                movem.l a2/a6,-(SP)

                move.l  a0,a2
                move.l  d0,(1*4,a2)

                move.l  #$00010001,d1           ; MEMF_PUBLIC|MEMF_CLEAR
                move.l  ($4).w,a6
                jsr     (-$C6,a6)               ; AllocMem()
                move.l  d0,(0*4,a2)             ; new_SPLower
                beq.b   .ErrorExit
                add.l   d0,(1*4,a2)             ; new_SPUpper

                move.l  (1*4,a2),a1
                movem.l (SP)+,d0-d1/a0
                movem.l d0-d1/a0,-(a1)
                move.l  a1,(2*4,a2)             ; new_SPReg

                cmp.w   #37,($14,a6)            ; LIB_VERSION
                bpl.b   .UseOS

                jsr     (-$78,a6)               ; Multitasking verbieten,
                sub.l   a1,a1                   ; damit kein Taskwechsel er-
                jsr     (-$126,a6)              ; folgt bevor wir fertig
                move.l  d0,a1                   ; sind

                move.l  ($3A,a1),d0             ; SPLower mit Eintrag im
                move.l  (0*4,a2),($3A,a1)       ; StackDataBuffer tauschen
                move.l  d0,(0*4,a2)

                move.l  ($3E,a1),d0             ; SPUpper tauschen
                move.l  (1*4,a2),($3E,a1)       ;
                move.l  d0,(1*4,a2)

                move.l  (2*4,a2),d0             ; SPReg ist in der Task-
                move.l  d0,($36,a1)             ; struktur nicht gltig,
                move.l  SP,(2*4,a2)             ; deshalb den SP in den
                                                ; Puffer eintragen

                move.l  d0,SP                   ; Stack tauschen!
                jsr     (-$7E,a6)
                bra.b   .Exit

.UseOS          move.l  a2,a0
                jsr     (-$2DC,a6)              ; StackSwap()

.Exit           moveq   #-1,d0
.ErrorExit      movem.l (SP)+,a2/a6
                rts




;****** AllocNewStack.asm/RestoreStack **************************************
;
;
;    NAME
;       RestoreStack -- restores a systemstack exchanged with
;               AllocNewStack()
;
;    SYNOPSIS
;       RestoreStack( StackDataBuffer)
;                     A0
;
;       ULONG   StackDataBuffer[3];
;
;    FUNCTION
;       The actual stack is exchanged with the one described by the
;       StackDataBuffer and the memoryblock is freed.
;
;    INPUTS
;       StackDataBuffer - pointer to the buffer initialized by
;               AllocNewStack()
;
;    RESULT
;       none
;
;    SEE ALSO
;       AllocNewStack()
;
;****************************************************************************


                XDEF    _RestoreStack

_RestoreStack:
                move.l  (2*4,a0),a1
                move.l  (SP)+,-(a1)             ; Register und RTS-Adresse
                movem.l a2/a6,-(a1)             ; auf dem Systemstack (!!)
                move.l  a1,(2*4,a0)             ; retten

                move.l  a0,a2
                move.l  ($4).w,a6

                cmp.w   #37,($14,a6)            ; LIB_VERSION
                bpl.b   .UseOS

                jsr     (-$78,a6)               ; Disable()
                sub.l   a1,a1
                jsr     (-$126,a6)              ; FindTask()
                move.l  d0,a0

                move.l  ($3A,a0),a1             ; SPLower
                move.l  (0*4,a2),($3A,a0)

                move.l  ($3E,a0),d0             ; SPUpper
                move.l  (1*4,a2),($3E,a0)

                move.l  (2*4,a2),($36,a0)       ; SPReg
                move.l  (2*4,a2),SP

                jsr     (-$7E,a6)               ; Enable()
                bra.b   .GoOn

.UseOS          jsr     (-$2DC,a6)              ; StackSwap()

                move.l  (0*4,a2),a1             ; Werte sind nur gltig,
                move.l  (1*4,a2),d0             ; wenn die OS-Routinen zum
.GoOn           sub.l   a1,d0                   ; Tauschen benutzt wurden!
                jsr     (-$D2,a6)               ; FreeMem()

                movem.l (SP)+,a2/a6
                rts


