/* Usenetstats: in SAS/C 6.3 (Amiga)
 *
 * Purpose: Provide statistics on USEnet usage
 *
 * Author: Andrew Clayton, June 13th 1992
 *
 * History:
 *            16/6/92  version 1.0 for Usenet. Based on Fidostats.
 *            4/10/92  version 1.1 - cleaned up code a lot
 *          22-Oct-93  re-compiled the source with SAS/C (P. Simons)
 *
 * Compiled: SC usenetstats.c LINK MATH=FFP
 *
 * Usage: Usenetstats <area> <filelist>
 *
 * Assumes that articles have been
 * concatenated together with Join.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <error.h>
#include <time.h>

#define TRUE         1
#define FALSE        0
#define MAXUSERS     4000       /* maximum number of users I can cope
                                 * with */
#define MAXFILENAME  130        /* maximum length of a filename */
#define MAXSTRING    256        /* max length of a string */
#define NAMELEN      41         /* max length of Usenet name */

#define WELCOME      "Usenetstats. Copyright 1992 David Andrew Clayton\n"
#define USAGE        "\nUsage: Usenetstats <area> <filename>\n" \
                     "  where  <area> is the Usenet Newsgroup that you want to scan\n" \
                     "  and    <filename> of filenames!\n"

struct Player {
        char name[NAMELEN];     /* username */
        char handle[NAMELEN];   /* handle */
        char messid[NAMELEN];   /* Message-ID */
        long bytecount;         /* total bytes */
        long quotes;            /* total bytes of quotes */
        int largest;            /* largest message */
        int smallest;           /* smallest message */
        int average;
        int number;             /* total # messages */
        int avgrank;            /* rank (determined later) */
        int byterank;
        int numrank;
}
aplayer;

 /* procedure declarations */

int main(int argc, char **argv);
void rextract(void);            /* open input file and collect data */
int emitstats(void);            /* writes raw stats to a file */
void getdata(FILE * mytarget);  /* get the data */
int getnames(char *targetstring,
             char *thisname,
             char *tempname);   /* get usernames */
void getbracedname(int anglebrace,
                   char *targetstring,
                   char *thisname,
                   char *tempname);     /* separate names */
void failed(char *why);         /* error exit */
int checkuser(char *target);    /* check if a user is already defined */
void condense(void);            /* do a sanity check on data */
void sortstats(void);           /* sort statistics */
void printstats(void);          /* print out the stats */
int sortsize(struct Player **,
             struct Player **); /* sort by size function */
int sortmsgs(struct Player **,
             struct Player **); /* sort by #msgs function */
int sortavgs(struct Player **,
             struct Player **); /* sort by #average function */
int getuser(char *name, char *tname);   /* get From: name */
int isQuotedLine(char *);       /* determine if a line is a quote */

 /* global variables */

char areaname[MAXFILENAME] = "x";       /* areaname to look for */
char mainfiles[MAXFILENAME];    /* filenames set up within program */
char outfiles[MAXFILENAME];
char particulars[MAXFILENAME];  /* name of file containing list! */
struct Player *allplayers[MAXUSERS], *alljunk;  /* set up our player
                                                 * structures */

int headerbytes = 0;

int totalmessages = 0;          /* total number of messages read */
int totalbytes = 0;             /* total bytes read */
int totalquotes = 0;            /* total amount of quoted text */
int totalplayers = 0;           /* number of different FROM players */
int totalto = 0;                /* number of different TO players */
int lastplayer = 0;
int singlemessages = 0;         /* users who only posted one message */

int largest = 0;                /* largest message yet found */
int bigavg = 0;
int mostto = 0;
int leastto = 0;
int mostfrom = 0;
int leastfrom = 0;

long timestart, timeend, tottime;       /* for time information */

char largestplayer[NAMELEN] = "Nobody";
char biggestavg[NAMELEN] = "NobodyHere";
char mostfromname[NAMELEN] = "NobodyHere";

FILE *areas;                    /* file handle for areas.bbs */
FILE *textfile;                 /* file handle for textfile(s) */

 /*
  * Main: Call other procedures, and have a nice day
  */
int main(int argc, char *argv[])
{
        timestart = time(&timestart);

        fprintf(stderr, WELCOME "\n%s", ctime(&timestart));

        if ((argc < 3) || (*argv[1] == '?')) {  /* no args */
                fputs(USAGE, stderr);
                return (0);
        }

        strcpy(areaname, argv[1]);      /* area we're doing stats on
                                         * (any string) */
        strcpy(particulars, strupr(argv[2]));   /* get filename */

        rextract();             /* extract data from supplied list of
                                 * files */

        condense();             /* pick out players who are the same. */

/*      emitstats();             * prelim stats */

        sortstats();            /* sort statistics */

        printstats();           /* print them out */

        timeend = time(&timeend);
        tottime = timeend - timestart;

        printf("\nIt took %ld seconds (%0.2f minutes) to do this task\n", tottime, (float) tottime / 60l);
/*      puts("\nAnother run of Usenetstats.  A program authored by Dac in 1992\n"); */
        return (0);             /* all done */
}

 /*
  * read and extract - rextract! open files, and get data
  */
void rextract(void)
{
        FILE *listfile;         /* file handle for particulars */
        FILE *tfile;            /* target file */
        char targetfile[MAXFILENAME];   /* filename storage */
        int charcount;

        listfile = fopen(particulars, "r");
        if (errno)
                failed("No listfile. \07x \07x");

        for (;;) {
                errno = 0;
                if (fgets(targetfile, sizeof(targetfile), listfile) == NULL)
                        break;
                fprintf(stderr, "Reading %s", targetfile);
                charcount = strlen(targetfile);
                targetfile[charcount - 1] = '\0';
                tfile = fopen(targetfile, "r");
                if (!errno)
                        getdata(tfile);
                fclose(tfile);
        }
}

 /*
  * Getdata: read the file, and extract _relevent_ data
  */
void getdata(FILE * filetarget)
{
        int linelength;         /* if I need it, I'll use it. */
        int msglength;          /* size of this message */
        int index = 0;          /* counter into array of structs */
        int header;
        int got = 0;
        int Quit;               /* a flag */
        char textline[MAXSTRING];       /* string storage */
        char thisname[NAMELEN]; /* name of person */
        char tempname[NAMELEN];
        char tempmsid[NAMELEN]; /* temporary message ident */
        char *thisnameptr = 0;
        char *tempnameptr = 0;
        char *strptr;
        char *tempptr;

        thisnameptr = thisname;
        tempnameptr = tempname;

        /*
         * Note: First time through the loop, nothing is in 'textline',
         * so put something in there to begin with.
         */
        strcpy(textline, "Begin!");

        /*
         * loop through file looking for 'Path:'
         */
        while (feof(filetarget) == 0) { /* while #1 */
                if (strncmp(textline, "Path: ", 6) == 0) {      /* if #1  only if start
                                                                 * of a message */
                        msglength = 0;
                        header = 0;
                        got = 0;
                        totalmessages += 1;
                        while ((textline[0] != '\n') && (!feof(filetarget))) {
                                header += strlen(textline);     /* increment number of
                                                                 * header bytes */
                                if ((textline[0] == 'F') && (textline[1] == 'r') && !got) {
                                        got = 1;
                                        getnames(textline, thisnameptr, tempnameptr);
                                }
                                if
                                        (
                                                (textline[0] == 'M') &  /* look for a line
                                                                         * starting with */
                                                (textline[7] == '-') &  /* Message-ID: in the
                                                                         * header */
                                                (textline[8] == 'I')
                                        ) {
                                        strptr = strrchr(textline, '@') + 1;
                                        tempptr = strrchr(textline, '>');
                                        tempptr[0] = '\0';
                                        strptr[41] = '\0';      /* just in case string
                                                                 * is too long */
                                        strcpy(tempmsid, strptr);
                                }
                                /* get next line */
                                fgets(textline, sizeof(textline), filetarget);  /* get next line */
                        }
                        totalbytes += header;
                        headerbytes += header;
                        if (!got)
                                getnames("From: idiot@dumbsite (Some Bozo With no \"From:\" field)", thisnameptr, tempnameptr);
                        index = getuser(thisname, tempname);
                        strcpy(allplayers[index]->messid, tempmsid);
                        Quit = FALSE;
                        fgets(textline, sizeof(textline), filetarget);
                        while (!Quit) { /* while2 */
                                linelength = strlen(textline);
/*                              strptr = strchr(textline, '>'); */
                                if (isQuotedLine(textline)) {
                                        allplayers[index]->quotes += linelength;
                                        totalquotes += linelength;
                                }
                                msglength += linelength;
                                fgets(textline, sizeof(textline), filetarget);
                                if ((strncmp(textline, "Path: ", 6) == 0) || feof(filetarget))
                                        break;
                        }       /* while #2 */
                        totalbytes += msglength;
                        allplayers[index]->bytecount += msglength;
                        if (msglength > allplayers[index]->largest)
                                allplayers[index]->largest = msglength;
                        if (allplayers[index]->smallest > msglength)
                                allplayers[index]->smallest = msglength;
                        if (msglength > largest) {
                                largest = msglength;
                                strcpy(largestplayer, allplayers[index]->name);
                        }
                }               /* if #1 */
                if (!got)       /* only increment textline if no Path:
                                 * line detected */
                        if (fgets(textline, sizeof(textline), filetarget) == NULL)
                                break;
        }                       /* while #1 */
}

 /*
  * getnames - extract username out of the From: string
  */
int getnames(char *targetstring, char *thisname, char *tempname)
{
        char *strptr;
        int anglebrace = 1;     /* using angle braces? */
        int valid = 0;          /* is a valid username? */

        strptr = strchr(targetstring, '<');
        if (strptr != NULL)
                valid = 1;

        if (!valid) {           /* not '<' */
                anglebrace = 0;
                strptr = strchr(targetstring, '(');
                if (strptr != NULL)
                        valid = 1;
        }

        if (!valid) {           /* neither bracket or brace */
                strncpy(thisname, &targetstring[0] + 6, 40);
                thisname[strlen(thisname) - 1] = '\0';
                strcpy(tempname, "no name specified");
                return 0;
        }

        getbracedname(anglebrace, targetstring, thisname, tempname);
        return (1);
}

 /*
  * getbracedname - extract the users site name and real name from
  * string
  */
void getbracedname(int anglebrace, char *targetstring,
                   char *thisname, char *tempname)
{
        char *strptr;
        char *otherptr;
        char openbrace, closebrace;
        int loop;

        (anglebrace) ? (openbrace = '<') : (openbrace = '(');
        (anglebrace) ? (closebrace = '>') : (closebrace = ')');

        /*
         * copy the users name/site e.g. dac@prolix.sserve
         */

        if (anglebrace) {       /* site name is in <machine@site.etc> */
                strptr = strchr(targetstring, '<');     /* '<' must exist */
                *strptr++;      /* skip bracket */
                otherptr = strchr(targetstring, '>');   /* '>' might not! */
                if (otherptr == NULL)
                        strcpy(thisname, "broken site");
                else
                        otherptr[0] = '\0';     /* erase bracket */
                if (strlen(strptr) > NAMELEN)
                        strptr[NAMELEN - 1] = '\0';
                strcpy(thisname, strptr);

                /* now to extract the username! */

                /* note strptr is pointing to one char past the '<' */

                strptr[-1] = '\0';      /* wicked! */
                strcpy(tempname, &targetstring[0] + 6); /* get handle */

                /* eradicate trailing spaces from handle */

                if (strlen(tempname) > 0)
                        for (loop = strlen(tempname); loop; loop -= 1)
                                if (tempname[loop - 1] == ' ' ||
                                    tempname[loop - 1] == '\n' ||
                                    tempname[loop - 1] == '\t')
                                        tempname[loop - 1] = '\0';
                                else
                                        break;

                if (strlen(tempname) == 0)
                        strcpy(tempname, "no name specified");
        }

        else {                  /* site name is not in braces at all */
                strncpy(thisname, &targetstring[0] + 6, NAMELEN);
                thisname[NAMELEN - 1] = '\0';   /* terminate string */
                strptr = strchr(thisname, openbrace);
                if (strptr != NULL)
                        *strptr = '\0';

                while (strrchr(thisname, ' ') != NULL)
                        thisname[strlen(thisname) - 1] = '\0';  /* remove trailing
                                                                 * spaces */

                if ((strlen(thisname) <= 2))    /* any site less than 2
                                                 * chars long is invalid
                                                 * anyway! */
                        strcpy(thisname, "no site specified");

                /* work out the user name (e.g. '(Andrew Clayton)') */

                strptr = strchr(targetstring, openbrace);       /* find brace */
                *strptr++;      /* skip brace */
                otherptr = strchr(targetstring, closebrace);    /* find closing brace */
                if (otherptr == NULL) { /* fucking idiots didn't put a
                                         * closing brace on their name */
                        strcpy(tempname, "a braceless fuck");
                        return;
                }
                otherptr[0] = '\0';     /* terminate string at brace */
                if (strlen(strptr) >= NAMELEN)
                        strptr[NAMELEN - 1] = '\0';     /* terminate long
                                                         * username */
                strcpy(tempname, strptr);       /* extract name */
                /* eradicate trailing spaces from handle */

                if (strlen(tempname) > 0)
                        for (loop = strlen(tempname); loop; loop -= 1)
                                if (tempname[loop - 1] == ' ' ||
                                    tempname[loop - 1] == '\n' ||
                                    tempname[loop - 1] == '\t')
                                        tempname[loop - 1] = '\0';
                                else
                                        break;

                if (strlen(tempname) == 0)
                        strcpy(tempname, "no name specified");
        }
        return;
}

 /*
  * getuser: gets username:
  */
int getuser(char *myname, char *tname)
{

        int myindex;            /* a counter */

        myindex = checkuser(myname);
        if (!myindex) {
                totalplayers += 1;
                myindex = totalplayers;
                if (totalplayers > MAXUSERS) {
                        fprintf(stderr, "totalplayers (%d)\n", totalplayers);
                        failed("Sorry: Not enough entries for number of Players");
                }
                if ((allplayers[myindex] = malloc(sizeof(struct Player))) == NULL)
                         failed("Insufficient memory to allocate another FROM user");

                strcpy(allplayers[myindex]->name, myname);
                strcpy(allplayers[myindex]->handle, tname);
                allplayers[myindex]->avgrank = 0;       /* initialise structure */
                allplayers[myindex]->byterank = 0;
                allplayers[myindex]->numrank = 0;
                allplayers[myindex]->number = 1;
                allplayers[myindex]->average = 0;
                allplayers[myindex]->quotes = 0;
                allplayers[myindex]->bytecount = 0;
                allplayers[myindex]->smallest = 65535;  /* kludge! fixed in
                                                         * getdata */
                allplayers[myindex]->largest = 0;
        }
        else {
                allplayers[myindex]->number += 1;
        }
        return (myindex);
}

 /*
  * Checkuser: Does user exist in database?
  */
int checkuser(char *target)
{
        int loop;               /* loop counter */

        if (stricmp(target, allplayers[lastplayer]->name) == 0)
                return (lastplayer);

        for (loop = 1; loop <= totalplayers; loop += 1) {
                if (stricmp(target, allplayers[loop]->name) == 0) {
                        lastplayer = loop;
                        return (loop);
                }
        }
        return (FALSE);         /* user not found */
}

 /*
  * condense; try to condense the list of users
  */
void condense(void)
{
        int count = 1;          /* how many iterations */
        int loop;               /* loop variable */
        int loop2;              /* another loop variable */

        while (count <= totalplayers) {
                for (loop = count + 1; loop <= totalplayers; loop += 1) {
                        if (strcmp(allplayers[count]->messid, allplayers[loop]->messid) == 0)
                                if (strcmp(allplayers[count]->handle, allplayers[loop]->handle) == 0) {
                                        allplayers[count]->number += allplayers[loop]->number;
                                        allplayers[count]->average += allplayers[loop]->average;
                                        allplayers[count]->quotes += allplayers[loop]->quotes;
                                        allplayers[count]->bytecount += allplayers[loop]->bytecount;
                                        if (allplayers[count]->smallest > allplayers[loop]->smallest)
                                                allplayers[count]->smallest = allplayers[loop]->smallest;
                                        if (allplayers[count]->largest < allplayers[loop]->largest)
                                                allplayers[count]->largest = allplayers[loop]->largest;
                                        for (loop2 = loop + 1; loop2 <= totalplayers; loop2 += 1)
                                                allplayers[loop2 - 1] = allplayers[loop2];      /* reorder list */
                                        totalplayers -= 1;
                                }
                }               /* for */
                count += 1;
        }                       /* while */
}

 /*
  * Emitstats: Write stats out to an intermediate file prior to sort
  */
int emitstats(void)
{
        FILE *mylfn;
        char filename[30] = "ram:currentrawstats";
        int loop;

        mylfn = fopen(filename, "w");
        if (errno) {
                perror("ram:currentrawstats - unable to open");
                return (0);
        }

        fprintf(mylfn, "%d  (total number of players)\n\n", totalplayers);
        fprintf(mylfn, "username           bytes,quotes,largest,smallest,number\n");

        for (loop = 1; loop <= totalplayers; loop++)
                fprintf(mylfn, "%50s%7d,%6d,%5d,%4d,%4d\n",
                        allplayers[loop]->name,
                        allplayers[loop]->bytecount,
                        allplayers[loop]->quotes,
                        allplayers[loop]->largest,
                        allplayers[loop]->smallest,
                        allplayers[loop]->number);

        fclose(mylfn);
}

 /*
  * Sortstats: Print out the collected stats
  */
void sortstats(void)
{
        int loop;

        /* calculate averages */

        for (loop = 1; loop <= totalplayers; loop += 1)
                allplayers[loop]->average = allplayers[loop]->bytecount / allplayers[loop]->number;

        /* calculate average message size ranking */
        qsort((char *) &allplayers[1], totalplayers, sizeof(struct Player *), sortavgs);

        for (loop = 1; loop <= totalplayers; loop += 1)
                allplayers[loop]->avgrank = loop;

        strcpy(biggestavg, allplayers[1]->name);
        bigavg = allplayers[1]->average;

        /* calculate most number of bytes posted ranking */
        qsort((char *) &allplayers[1], totalplayers, sizeof(struct Player *), sortsize);

        for (loop = 1; loop <= totalplayers; loop += 1)
                allplayers[loop]->byterank = loop;

        /* calculate most number of messages ranking */
        qsort((char *) &allplayers[1], totalplayers, sizeof(struct Player *), sortmsgs);

        for (loop = 1; loop <= totalplayers; loop += 1)
                allplayers[loop]->numrank = loop;

        mostfrom = allplayers[1]->number;
        strcpy(mostfromname, allplayers[1]->name);

        for (loop = 1; loop <= totalplayers; loop += 1)
                if (allplayers[loop]->number == 1)
                        break;
        singlemessages = loop;
}

 /*
  * sortsize : routine used by Qsort to sort longs ... NOTE: this is a
  * REVERSE sort!
  */
int sortsize(struct Player **one, struct Player **two)
{
        return ((*two)->bytecount - (*one)->bytecount);
}

 /*
  * sortmsgs : routine used by Qsort to sort ints ... NOTE: this is a
  * REVERSE sort!
  */
int sortmsgs(struct Player **one, struct Player **two)
{
        if ((*two)->number == (*one)->number) {
                if ((*two)->bytecount > (*one)->bytecount)
                        return (1);
                else
                        return (-1);
        }
        else
                return ((*two)->number - (*one)->number);
}

 /*
  * sortavgs : routine used by Qsort to sort ints ... NOTE: this is a
  * REVERSE sort!
  */
int sortavgs(struct Player **one, struct Player **two)
{
        return ((*two)->average - (*one)->average);
}

 /*
  * Printstats: Print out the collected stats
  */
void printstats(void)
{
        int loop = 0;           /* loop counter */
        int count = 0;
        char sep;               /* separator */
        double avgmessages = 0;
        double avgbytesmsg = 0;
        double avgbytesusr = 0;
        double pcentbyte = 0;
        double pcentmsgs = 0;
        double pcentquote = 0;
        double ftotalm = 0;
        double ftotalb = 0;
        double ftotalp = 0;
        double totbyte = 0;
        double totquote = 0;
        double tothead = 0;

        printf(" Usenet statistics ---  Newsgroup = %s\n Runtime: %s\n\n", areaname, ctime(&timestart));

        ftotalm = (double) totalmessages;
        ftotalb = (double) (totalbytes - headerbytes);
        ftotalp = (double) totalplayers;
        totbyte = (double) totalbytes;
        totquote = (double) totalquotes;
        tothead = (double) headerbytes;
        avgmessages = (double) (ftotalm / ftotalp);
        avgbytesmsg = (double) (ftotalb / ftotalm);
        avgbytesusr = (double) (ftotalb / ftotalp);
        pcentbyte = (double) ((tothead / totbyte) * 100.0);     /* this var is reused
                                                                 * later! */
        pcentquote = (totquote / totbyte) * 100.0;      /* so is this one. */

        printf("Total Messages          %7ld\n"
               "Total bytes             %7ld\n"
               "Headerbytes             %7ld"
               " %2.2lf%% of total bytes\n"
               "Amount of quoted text   %7ld"
               " %2.2lf%% of total bytes\n",
               totalmessages,
               totalbytes,
               headerbytes,
               pcentbyte,
               totalquotes,
               pcentquote);

        printf("Total players           %7ld\n"
               "Single article posters  %7ld\n"
               "Avg Msgs/User           %7.0lf\n"
            "Avg Bytes/Msg           %7.0lf   (not including header)\n"
           "Avg Bytes/User          %7.0lf   (not including header)\n",
               totalplayers,
               (totalplayers - singlemessages) + 1,
               avgmessages,
               avgbytesmsg,
               avgbytesusr);

        printf("\nLargest msg       %5d   %s"
               "\nLargest Avg       %5d   %s"
               "\nMost msgs from    %5d   %s\n\n",
               largest, largestplayer,
               bigavg, biggestavg,
               mostfrom, mostfromname);

        puts("Notes: The originality field is the percent ratio of original text,");
        puts("       minus the amount of quoted text. A line is deemed to be a quote");
        puts("       if it has a '>' character in the first six columns.\n");
        puts("       Some users don't specify a 'handle', and I've given these people");
        puts("       the generic title of 'no name specified'.\n");
        puts("       Some users post from many different sites, and some users like to");
        puts("       use different 'handles'. I've tried to cope with this, but in some");
        puts("       cases the code fails miserably.\n");

        puts("==============================================================================");
        puts("                    |Kilobytes Sent |Messages Sent | Average |  Max   Min| Ori");
        puts("                    |Total   Rank   |       Rank   |  Msg    |  Msg   Msg|gina");
        puts("Username            |KByte Pos   %  | Num Pos   %  | Size Rnk| Size  Size|lity");
        puts("====================|===== === =====|==== === =====|===== ===|===== =====|====");

        for (loop = 1; loop < singlemessages; loop += 1) {
                pcentbyte = ((double) allplayers[loop]->bytecount / ftotalb) * 100.0;
                pcentmsgs = ((double) allplayers[loop]->number / ftotalm) * 100.0;
                pcentquote = 100.0 - (((double) allplayers[loop]->quotes / (double) allplayers[loop]->bytecount) * 100.0);

/*    printf("%s%0.59s\n%20.20s%7d %3d %2.0f% %3d %3d %2.0f%  %5d %3d  %5d  %5d %3.0f%\n", */
                printf("%-35.35s%s\n%-20.20s %5.1f %3d %4.1f | %3d %3d %4.1f |%5d %3d|%5d %5d|%3.0f%%\n",

                       allplayers[loop]->handle,
                       " |              |         |           |",
                       allplayers[loop]->name,

                       (double) (allplayers[loop]->bytecount / 1024.0),
                       allplayers[loop]->byterank,
                       pcentbyte,

                       allplayers[loop]->number,
                       allplayers[loop]->numrank,
                       pcentmsgs,

                       allplayers[loop]->average,
                       allplayers[loop]->avgrank,

                       allplayers[loop]->largest,
                       allplayers[loop]->smallest,
                       pcentquote);
        }
        printf("\n Users posting only a single article in the reporting period (%d users)\n\n", (totalplayers - singlemessages) + 1);
        for (loop = singlemessages, count = 1; loop <= totalplayers; loop += 1, count += 1) {
                ((count % 3) != 0) ? (sep = '|') : (sep = ' ');
                if (strcmp(allplayers[loop]->handle, "no name specified") == 0)
                        printf("%20.20s  %c  ", allplayers[loop]->name, sep);
                else
                        printf("%20.20s  %c  ", allplayers[loop]->handle, sep);
                if ((count % 3) == 0)
                        putchar('\n');
        }
        putchar('\n');
}


 /*
  * Determine if a line is a quote
  */
int isQuotedLine(char *line)
{
        char *textline = line;

        if (textline[0] == ':')         /* Recognize: ": text" */
                return 1;

        if (!strncmp(textline, "->", 2)) /* Recognize: "->text" */
                return 1;

        while (isspace(*textline))
                if (*textline == '\n')
                        return 0;
                else
                        textline++;
        if (*textline == '>')           /* Recognize: " > text" */
                return 1;

        while (!isspace(*textline))
                if (*textline == '\n')
                        return 0;
                else
                        textline++;
        if (*(textline-1) == '>')           /* Recognize: " AB> text" */
                return 1;

        return 0;               /* No Quote :-> */
}


void failed(char *why)
{
        puts(why);
        exit(20);               /* kill us dead */
}


