/*
 *      $Filename: ConfigFiles.c $
 *      $Revision: 1.22 $
 *      $Date: 1994/10/25 15:43:24 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: ConfigFiles.c,v 1.22 1994/10/25 15:43:24 simons Exp simons $
 *
 */


/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <string.h>
#include <proto/utility.h>
#include <exec/types.h>
#include <proto/dos.h>

#include "libraries/netsupport.h"
#include "NetSupportLibrary.h"
#include "MemoryHandling.h"
#include "LockFiles.h"
#include "ConfigFiles.h"
#include "SupportRoutines.h"
#include "Residents.h"

/************************************* Defines ************/

/************************************* Prototypes *********/

static const char __RCSId[] = "$Id: ConfigFiles.c,v 1.22 1994/10/25 15:43:24 simons Exp simons $";

/**************************************************************************
 *                                                                        *
 * SEKTION: Public library functions                                      *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/GetConfigEntry *********************************
*
*
*    NAME
*       GetConfigEntry -- return a configuration entry
*
*    SYNOPSIS
*       entry = GetConfigEntry(filename, keyword, default);
*       D0                     A0        A1       A2
*
*       STRPTR  entry;
*       STRPTR  filename;
*       STRPTR  keyword;
*       STRPTR  default;
*
*    FUNCTION
*       This routine is designed to replace the old Dillon UUCP config
*       routines and to perform some simple configuration issues.
*
*       It checks if the provided keyword is available as local or
*       global system variable and eventually returned its contents. If
*       such a variable isn't set, it loads specified configfile and
*       searches for the keyword, returning the specified parameter.
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the configfile in case no variable is set.
*
*               When <filename> is NULL, the routine uses the configfile,
*               you specified as 'MasterConfig' in the library's own
*               configfile. The default is 'UULib:Config'.
*
*       keyword - Pointer to a keyword string. All comparisons are
*               case-independent!
*
*       default - pointer to a string containing the default parameter,
*               which is returned in case, the keyword can't be found.
*
*    RESULT
*       GetConfigEntry() returns a pointer to a static buffer, holding
*       the provided parameter. Please take note that the contents of
*       this buffer is lost, when GetConfigEntry() or GetConfig() is
*       called the next time. You MUST not free or modify this
*       library-internal buffer!
*
*       If no config entry matching your keyword was available, the
*       provided default is returned.
*
*    NOTE
*       GetConfigEntry() does NOT lock the file in order to avoid
*       deadlocks.
*
*    CONFIGFILE SYNTAX
*       A valid config file may contain one keyword per line. Keywords
*       MUST begin at the first column. The parameter can be separated
*       from the keyword using either space(s) or tab(s).
*
*       GetConfigEntry() skips all leading blanks or tabs until the
*       first "non-blank" character. Then it returns the whole line
*       until the return, truncating any blanks or tabs at the end of
*       the line. If you specify a keyword alone on a line,
*       GetConfigEntry() will return a pointer to an empty string.
*
*       Lines starting with a '#' are commentlines and will be ignored.
*
*
*       Example:
*               KEYWORD parameter
*               FOO     bar
*               # comment
*               FO   " O  B A R "
*
*       GetConfigEntry does not parse the parameter and therefore does
*       NOT remove the double-quotes '"' in the last line!
*
*    SEE ALSO
*       GetConfig(), ParseConfigFile()
*
***************************************************************************
*
*/

STRPTR __asm GetConfigEntry(register __a0 STRPTR filename,
                            register __a1 STRPTR keyword,
                            register __a2 STRPTR def)
{
        STRPTR parameter, filebuffer;
        void **GCEntryBuffer = &((GetStaticBuffer())->nspsb_GCEntryBuffer);


        /* Free the buffer we used last time */

        if (*GCEntryBuffer) {
                NSPFreeMemPooled(*GCEntryBuffer);
                *GCEntryBuffer = NULL;
        }

        /* If no filename is given, we access the MasterConfig file. */

        if (filename)
                filebuffer = LoadResident(filename);
        else
                filebuffer = LoadResident(GetConfig(GetNSPConfigName(), "MasterConfig", NULL, "UULib:Config"));


        /* If the file has be loaded successfully, find the keyword
         * and returen the parameter.
         */

        if (filebuffer) {
                if (parameter = GetKeywordLine(keyword, filebuffer))
                        parameter = InterpretKeywordLine(parameter);

                if (parameter) {        /* track buffer for later usage */
                        *GCEntryBuffer = parameter;
                        return parameter;
                }
        }

        return def;     /* no buffer allocated--return default */
}


/****** netsupport.library/SetConfigEntry *********************************
*
*
*    NAME
*       SetConfigEntry -- set a configuration entry
*
*    SYNOPSIS
*       success = SetConfigEntry(filename, keyword, parameter);
*       D0                       A0        A1       A2
*
*       LONG    success;
*       STRPTR  filename;
*       STRPTR  keyword;
*       STRPTR  parameter;
*
*    FUNCTION
*       This routine can be used to set a certain entry in a config
*       file. SetConfigEntry() will load the file and look for the
*       specified keyword. If the keyword can be found, it will replace
*       the parameter with the provided one and write the file back.
*
*       When the keyword couldn't be found, SetConfigEntry() will append
*       the keyword plus parameter at the end of the file.
*
*       Please read GetConfigEntry() for a brief description of the
*       configfile format.
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the configfile in case no variable is set.
*
*               When <filename> is NULL, the routine uses the configfile,
*               you specified as 'MasterConfig' in the library's own
*               configfile. The default is 'UULib:Config'.
*
*       keyword - Pointer to a keyword string. All comparisons are
*               case-independent!
*
*       parameter - pointer to a string containing the parameter to be
*               set behind the keyword in the configfile.
*
*    RESULT
*       SetConfigEntry() either TRUE (!0L) or FALSE (0L) for success/
*       failure.
*
*    NOTE
*       The specified configfile MUST at least exist or the routine will
*       fail!
*
*       SetConfigEntry() does NOT lock the file in order to avoid
*       deadlocks.
*
*    SEE ALSO
*       GetConfigEntry(), GetConfig()
*
***************************************************************************
*
*/

LONG __asm SetConfigEntry(register __a0 STRPTR filename,
                          register __a1 STRPTR keyword,
                          register __a2 STRPTR parameter)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        BPTR fh;
        STRPTR filebuffer, entry;
        LONG rc = 0L;


        /* If no filename is given, we use the MasterConfig file. */

        if (filename)
                filebuffer = LoadResident(filename);
        else
                filebuffer = LoadResident(GetConfigEntry(GetNSPConfigName(), "MasterConfig", "UULib:Config"));


        /* If the file has be loaded successfully, find the keyword
         * and returen the parameter.
         */

        if (filebuffer) {
                LockFile(filename);
                if (entry = GetKeywordLine(keyword, filebuffer)) {
                        if (fh = Open(filename, MODE_NEWFILE)) {
                                Write(fh, filebuffer, (entry-filebuffer));
                                FPuts(fh, keyword);
                                FPutC(fh, ' ');
                                FPuts(fh, parameter);
                                while (*entry != '\n' && *entry != '\0')
                                        entry++;
                                if (entry == '\0')
                                        FPutC(fh, '\n');
                                else {
                                        Flush(fh);
                                        Write(fh, entry, strlen(entry));
                                }
                                Close(fh);
                                rc = 1;
                        }
                }
                else
                {
                        if (fh = Open(filename, MODE_READWRITE)) {
                                Seek(fh, -1L, OFFSET_END);
                                if (FGetC(fh) != '\n')
                                        FPutC(fh, '\n');
                                FPuts(fh, keyword);
                                FPutC(fh, ' ');
                                FPuts(fh, parameter);
                                FPutC(fh, '\n');
                                Close(fh);
                                rc = 1;
                        }
                }
                UnLockFile(filename);
        }

        return rc;
}

/****** netsupport.library/GetConfig **************************************
*
*    NAME
*       GetConfig -- return a configuration entry or system variable
*
*    SYNOPSIS
*       entry = GetConfig(filename, keyword, dosvarname, default);
*       D0                A0        A1       A2          A3
*
*       STRPTR  entry;
*       STRPTR  filename;
*       STRPTR  keyword;
*       STRPTR  dosvarname;
*       STRPTR  default;
*
*    FUNCTION
*       GetConfig() does pretty much the same as GetConfigEntry(),
*       except that this routine is able to recognize local and global
*       DOS variables.
*
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the configfile in case no variable is set.
*
*               When <filename> is NULL, the routine uses the configfile,
*               you specified as 'MasterConfig' in the library's own
*               configfile. The default is 'UULib:Config'.
*
*       keyword - Pointer to a keyword string. All comparisons are
*               case-independent!
*
*       dosvarname - pointer to a name of the local shell- and global
*               enviroment variable to check. When this parameter is
*               NULL, <keyword> is used. However, some purposes may
*               require a different variable name than config file
*               keyword.
*
*       default - pointer to a string containing the default parameter,
*               which is returned in case, the keyword can't be found.
*
*    RESULT
*       GetConfig() returns a pointer to a static buffer, holding the
*       provided parameter. Please take note that the contents of this
*       buffer is lost, when GetConfig() or GetConfigEntry() is called
*       the next time. You MUST not free or modify this library-internal
*       buffer!
*
*       If no config entry matching your keyword was available, the
*       provided default is returned.
*
*    NOTE
*       GetConfig() does NOT lock the file in order to avoid deadlocks.
*
*    SEE ALSO
*       GetConfigEntry(), ParseConfigFile()
*
***************************************************************************
*
*/

STRPTR __asm GetConfig(register __a0 STRPTR filename,
                       register __a1 STRPTR keyword,
                       register __a2 STRPTR dosvarname,
                       register __a3 STRPTR def)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        void **GCEntryBuffer = &(GetStaticBuffer()->nspsb_GCEntryBuffer);


        /* Free the buffer we used last time */

        if (*GCEntryBuffer) {
                NSPFreeMemPooled(*GCEntryBuffer);
                *GCEntryBuffer = NULL;
        }

        /*
         * First check local and enviroment variables, if we can't find
         * something like that, look into the config file.
         */

        if (!(*GCEntryBuffer = NSPAllocMemPooled(256L, 0L)))
                return def;

        if (GetVar((dosvarname) ? dosvarname : keyword, *GCEntryBuffer, 256L, 0L) <= 0L) {
                NSPFreeMemPooled(*GCEntryBuffer);
                *GCEntryBuffer = GetConfigEntry(filename, keyword, NULL);
        }

        return ((*GCEntryBuffer) ? (STRPTR) *GCEntryBuffer : def);
}


/**************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/GetNSPConfigName *******************************
*
*    NAME
*       GetNSPConfigName -- return the name and path of the libraries
*               own config file.
*
*    SYNOPSIS
*       name = GetNSPConfigName(void);
*       D0
*
*       STRPTR  name;
*
*    FUNCTION
*       This routine tries to determine which path should be used for the
*       libraries own config file. You can put the file anywhere you want
*       and set the variable "NSPConfig" to the path and name of the file
*       or "S:NSPConfig" will be assumed as default.
*
*    INPUTS
*       none
*
*    RESULT
*       Name and path of the configfile. If the routine fails due to
*       memory lack, NULL is returned.
*
*    NOTE
*       You must not free or modify the returned pointer!
*
*    SEE ALSO
*
***************************************************************************
*
*/

STRPTR __asm GetNSPConfigName(void)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct StaticBuffer *sb;

        if (!(sb = GetStaticBuffer()))
                return NULL;

        if (!(sb->nspsb_NSPConfigName) && !(sb->nspsb_NSPConfigName = NSPAllocMemPooled(1024, 0L)))
                return NULL;

        if (GetVar("NSPConfig", sb->nspsb_NSPConfigName, 1024-1, 0L) == -1L)
                StringCopy(sb->nspsb_NSPConfigName, "S:NSPConfig");

        return sb->nspsb_NSPConfigName;
}


STRPTR __asm GetKeyword(register __a0 STRPTR keyword,
                        register __a1 STRPTR buffer)
{
        STRPTR parameter;

        if (parameter = GetKeywordLine(keyword, buffer))
                parameter = InterpretKeywordLine(parameter);


        /* If GetKeywordLine() didn't find the keyword, return a pointer
         * to the provided default. */

        if (!(*parameter))
                parameter = &keyword[strlen(keyword)+1];

        return parameter;
}


STRPTR __asm GetKeywordLine(register __a0 STRPTR keyword,
                            register __a1 STRPTR buffer)
{
        struct UtilityBase *UtilityBase = NetSupportBase->nsp_UtilityBase;
        int keywdlen;

        if (!keyword || !(keywdlen = strlen(keyword)))
                return NULL;            /* no keyword provided */

        while (*buffer) {
                if (!Strnicmp(keyword, buffer, keywdlen) && (buffer[keywdlen] == ' '
                                                             || buffer[keywdlen] == '\t'
                                                             || buffer[keywdlen] == '\n'
                                                            )
                   )
                        break;

                while(*buffer != '\0' && *buffer != '\n')
                        buffer++;       /* search for '\0' or '\n' */

                while (*buffer == '\n')
                        buffer++;       /* skip '\n' and following empty
                                         * lines. */
        }

        return (*buffer) ? buffer : NULL;
}

STRPTR __asm InterpretKeywordLine(register __a0 STRPTR keywordline)
{
        STRPTR parameter;
        int i;

        while (*keywordline != ' ' && *keywordline != '\t' && *keywordline != '\n')
                keywordline++;  /* skip keyword */

        while (*keywordline == ' ' || *keywordline == '\t')
                keywordline++;  /* skip leading blanks or tabs */

        for (i = 0; keywordline[i] != '\n' && keywordline[i] != '\0'; i++)
                ;               /* find end of line */

        if (i) {

                while (keywordline[i-1] == ' ' || keywordline[i-1] == '\t')
                        i--;            /* remove trailing blanks or tabs */

                if (parameter = NSPAllocMemPooled(i+1, 0L)) {
                        parameter[i] = '\0';    /* send end of string */
                        while (i--)             /* copy string to keywordline */
                                parameter[i] = keywordline[i];
                }
        }
        else
                if (parameter = NSPAllocMemPooled(1, 0L))
                        *parameter = '\0';

        return parameter;
}


