/*
 *      $Filename: LockFiles.c $
 *      $Revision: 1.17 $
 *      $Date: 1994/05/24 21:12:09 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: LockFiles.c,v 1.17 1994/05/24 21:12:09 simons Exp simons $
 *
 */


/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <strings.h>

#include <dos/dos.h>
#include <exec/memory.h>
#include <proto/dos.h>
#include <proto/exec.h>

#include "libraries/netsupport.h"
#include "LockFiles.h"
#include "NetSupportLibrary.h"
#include "MemoryHandling.h"
#include "SupportRoutines.h"

/************************************* Defines ************/

/************************************* Prototypes *********/
static const char __RCSId[] = "$Id: LockFiles.c,v 1.17 1994/05/24 21:12:09 simons Exp simons $";


/**************************************************************************
 *                                                                        *
 * SEKTION: Public library functions                                      *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/LockFile ***************************************
*
*    NAME
*       LockFile -- lock a file before actually accessing it
*
*    SYNOPSIS
*       success = LockFile(filename);
*       D0                 A0
*
*       LONG    success;
*       STRPTR  filename;
*
*    BACKGROUND
*       In a multitasking enviroment, like the Amiga is, it is important
*       to coordinate and syncronize the usage of the available
*       resources. This includes devices, memory and, of course, files.
*       What happens, if your program needs a certain config file, while
*       another program is reading/writing the file? You're attempt will
*       fail, the access will be denied. Now things become difficult.
*
*       Say, you just noticed that a file access failed. What should you
*       do? Maybe the file is free in a few seconds, maybe you can do
*       some other things while waiting for the file to become
*       available.
*
*       Using the libraries locking mechanism solves many problems for
*       you. Most packages have recognized this problem and use some
*       rather strange file locking mechanism, like Dillon UUCP 1.15.
*       or the OwnDevUnit.Library for this purpose. Well, these
*       mechanism had several disadvantages: They were not really
*       designed for files!
*
*       Both of them were not able to recognize that "UUMAIL:foobar" and
*       "UUCP:Mail/foobar" is the same file! And if you'd just used the
*       filenames without paths (what many authors did to come around
*       the above problem), a lock to "UUMAIL:foobar" would fail if
*       "ARCHIVE:foobar" was locked. All this problems are gone with
*       LockFile().
*
*       Forget about all this pseudo-names like "LOG-UPDATE.LOCK" and
*       similar names, lock the complete filename, including path!
*       LockFile() can handle all those situations, using SameLock() to
*       determine of the same file is access with two different paths.
*
*       Remember: Use the same filename to lock the file, you use for
*       the actual Open() call, or the mechanism will break!
*
*    FUNCTION
*       LockFile() determines the complete path to the file you
*       specified and tests wether this file is locked by some other
*       process and eventually waits for the file to become available.
*
*       UnLockFile() has to be called when the file isn't required
*       anymore.
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the file.
*
*    RESULT
*       LockFile() returns TRUE if the file could be locked
*       successfully. If an error occurs, FALSE will be returned.
*
*    SEE ALSO
*       LockFileAttempt(), IsFileLocked(), UnLockFile(), UnLockFiles()
*
***************************************************************************
*
* Currently the library's mechanism isn't implemented, that's why I stick
* stick with Dillon's LockFile(). Additionally this has the advantage that
* the library (while in beta stade) doesn't break the existing software.
*
*/

LONG __asm LockFile(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        STRPTR buffer;
        struct List *list;
        struct LockNode *node;
        struct StaticBuffer *staticbuf;
        BPTR fh;

        if (!(staticbuf = GetStaticBuffer()))
                return 0L;

        if (!(buffer = MakeLockName(filename)))
                return 0L;

        if (!staticbuf->nspsb_LockedFileList)
                if (list = NSPAllocMemPooled(sizeof(struct List), MEMF_ANY)) {
                        NewList(list);
                        staticbuf->nspsb_LockedFileList = list;
                }
                else {
                        NSPFreeMemPooled(buffer);
                        return 0L;
                }

        if (FindName(staticbuf->nspsb_LockedFileList, buffer)) {
                NSPFreeMemPooled(buffer);
                return 0L;
        }

        if (node = NSPAllocMemPooled(sizeof(struct LockNode), MEMF_ANY|MEMF_CLEAR)) {
                while (!(fh = Open(buffer, MODE_NEWFILE)))
                        Delay(3*50);
                node->node.ln_Name = buffer;
                node->filehandle = fh;
                AddHead(staticbuf->nspsb_LockedFileList, (struct Node *) node);
        }
        else {
                NSPFreeMemPooled(buffer);
                return 0L;
        }

        return 1L;
}


/****** netsupport.library/UnLockFile *************************************
*
*    NAME
*       UnLockFile -- free an previously locked file
*
*    SYNOPSIS
*       UnLockFile(filename);
*                  A0
*
*       STRPTR  filename;
*
*    FUNCTION
*       UnLockFile() releases a lock on a file, previously obtained
*       using LockFile().
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the file.
*
*    RESULT
*       none
*
*    SEE ALSO
*       LockFile(), LockFileAttempt(), UnLockFiles()
*
***************************************************************************
*
*/

void __asm UnLockFile(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        STRPTR buffer;
        struct LockNode *node;
        struct StaticBuffer *staticbuf;

        if (!(staticbuf = GetStaticBuffer()))
                return;

        if (!(buffer = MakeLockName(filename)))
                return;

        if (!staticbuf->nspsb_LockedFileList)
                return;

        if (node = (struct LockNode *) FindName(staticbuf->nspsb_LockedFileList, buffer)) {
                Close(node->filehandle);
                DeleteFile(buffer);
                Remove((struct Node *) node);
                NSPFreeMemPooled(node->node.ln_Name);
                NSPFreeMemPooled(node);
        }
        NSPFreeMemPooled(buffer);
}



/****** netsupport.library/UnLockFiles ************************************
*
*    NAME
*       UnLockFiles -- free all files locked by a process
*
*    SYNOPSIS
*       UnLockFiles();
*
*    BACKGROUND
*       Please refer to LockFile() for a brief description.
*
*    FUNCTION
*       UnLockFiles() releases all filelocks obtained the by calling
*       process. This can be useful in error situations.
*
*    INPUTS
*       none
*
*    RESULT
*       none
*
*    SEE ALSO
*       LockFile(), LockFileAttempt(), UnLockFile()
*
***************************************************************************
*
*/

void __asm UnLockFiles(void)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct LockNode *node;
        struct StaticBuffer *staticbuf;

        if (!(staticbuf = GetStaticBuffer()))
                return;

        if (!staticbuf->nspsb_LockedFileList)
                return;

        while (!IsListEmpty(staticbuf->nspsb_LockedFileList)) {
                node = (struct LockNode *) RemHead(staticbuf->nspsb_LockedFileList);
                Close(node->filehandle);
                DeleteFile(node->node.ln_Name);
                NSPFreeMemPooled(node->node.ln_Name);
                NSPFreeMemPooled(node);
        }
}



/****** netsupport.library/LockFileAttempt ********************************
*
*    NAME
*       LockFileAttempt -- try to lock a file
*
*    SYNOPSIS
*       success = LockFileAttempt(filename);
*       D0                        A0
*
*       LONG    success;
*       STRPTR  filename;
*
*    FUNCTION
*       LockFileAttempt() does mainly the same as LockFile(). The only
*       difference is, that LockFileAttempt() does not wait for the file
*       to become available, but returns immediately.
*
*       If the locking attempt was successful, UnLockFile() has to be
*       called when the file isn't required anymore.
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the file.
*
*    RESULT
*       LockFile() returns TRUE if the file could be locked
*       successfully. If an error occurred, or the file is locked
*       already, FALSE is returned.
*
*    NOTE
*       Lockfile() is able to recognize, if the same file is accessed
*       using two different paths, like "SYS:file" / "WB_2.x:file" for
*       example. This is archived using the system call SameLock().
*       However, due to this feature, LockFile() has to store the DOS
*       lock of an already locked file, meaning that the file has to
*       be created if it doesn't exist already.
*
*    SEE ALSO
*       LockFile(), IsFileLocked(), UnLockFile(), UnLockFiles()
*
*
***************************************************************************
*
*/

LONG __asm LockFileAttempt(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        STRPTR buffer;
        struct List *list;
        struct LockNode *node;
        struct StaticBuffer *staticbuf;
        BPTR fh;

        if (!(staticbuf = GetStaticBuffer()))
                return 0L;

        if (!(buffer = MakeLockName(filename)))
                return 0L;

        if (!staticbuf->nspsb_LockedFileList)
                if (list = NSPAllocMemPooled(sizeof(struct List), MEMF_ANY)) {
                        NewList(list);
                        staticbuf->nspsb_LockedFileList = list;
                }
                else {
                        NSPFreeMemPooled(buffer);
                        return 0L;
                }

        if (!(FindName(staticbuf->nspsb_LockedFileList, buffer)))
                if (node = NSPAllocMemPooled(sizeof(struct LockNode), MEMF_ANY|MEMF_CLEAR))
                        if (fh = Open(buffer, MODE_NEWFILE)) {
                                node->node.ln_Name = buffer;
                                node->filehandle = fh;
                                AddHead(staticbuf->nspsb_LockedFileList, (struct Node *) node);
                                return 1L;
                        }
                        else
                                NSPFreeMemPooled(node);

        /* File is already locked */

        NSPFreeMemPooled(buffer);
        return 0L;
}


/****** netsupport.library/IsFileLocked ***********************************
*
*    NAME
*       IsFileLocked -- returns the (lock-)status of a file
*
*    SYNOPSIS
*       boolean = IsFileLocked(filename);
*       D0                     A0
*
*       LONG    boolean;
*       STRPTR  filename;
*
*    BACKGROUND
*       Please refer to LockFile() for a brief description.
*
*    FUNCTION
*       Sometimes you want to know wether a file is locked or not,
*       without actually getting the exclusive access on the file--so
*       this is the routine to use.
*
*    INPUTS
*       filename - Pointer to a string, containing the path and filename
*               to access the file.
*
*    RESULT
*       IsFileLocked() returns either ZERO (0L) for an unlocked file,
*       any positive value for 'file is locked' or a negative value for
*       indicating an error.
*
*    NOTE
*       Lockfile() is able to recognize, if the same file is accessed
*       using two different paths, like "SYS:file" / "WB_2.x:file" for
*       example. This is archived using the system call SameLock().
*       However, due to this feature, LockFile() has to store the DOS
*       lock of an already locked file, meaning that the file has to
*       be created if it doesn't exist already.
*
*    SEE ALSO
*       LockFile(), LockFileAttempt()
*
***************************************************************************
*
*/

LONG __asm IsFileLocked(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        BPTR fh;
        STRPTR buffer;
        struct StaticBuffer *staticbuf;
        LONG success = 0;

        if (!(staticbuf = GetStaticBuffer()))
                return -1;

        if (!(buffer = MakeLockName(filename)))
                return -1;

        if (!staticbuf->nspsb_LockedFileList || !(success = (LONG) FindName(staticbuf->nspsb_LockedFileList, buffer))) {
                if (fh = Open(buffer, MODE_NEWFILE)) {
                        Close(fh);
                        DeleteFile(buffer);
                }
                else
                        success = 1;
        }
        NSPFreeMemPooled(buffer);

        return success;
}


/**************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/

STRPTR MakeLockName(STRPTR filename)
{
        STRPTR buffer;

        if (!(buffer = NSPAllocMemPooled(72, MEMF_ANY)))
                return 0;

        StringCopy(buffer, "T:");
        StringCat(buffer, FindFilePart(filename));
        StringCat(buffer, ".LOCK");

        return buffer;
}


STRPTR FindFilePart(STRPTR filename)
{
        STRPTR file;
        ULONG len;

        /*
         * Strip quotes!
         */
        len = strlen(filename);
        if (*filename == '"' && filename[len-1] == '"') {
                filename[len-1] = '\0';
                filename++;
        }


        /*
         * Strip path!
         */
        file = filename;
        while (*filename) {
                switch (*filename) {
                  case ':':
                  case '/':
                        file = filename+1;
                  default:
                        filename++;
                }
        }

        return file;
}

