/*
 *      $Filename: POpen.c $
 *      $Revision: 2.5 $
 *      $Date: 1994/05/24 21:12:09 $
 *
 *      Copyright (C) 1994 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: POpen.c,v 2.5 1994/05/24 21:12:09 simons Exp simons $
 *
 */


/************************************* includes ***********/
#include <string.h>
#include <exec/memory.h>
#include <exec/execbase.h>
#include <dos/dos.h>
#include <dos/dostags.h>
#include <dos/dosextens.h>
#include <proto/dos.h>
#include <proto/exec.h>

#include "libraries/netsupport.h"
#include "TmpFiles.h"
#include "SupportRoutines.h"
#include "MemoryHandling.h"
#include "NetSupportLibrary.h"

/************************************* structures *********/
struct PipeStruct {
        struct Message ps_PipeMsg;
        STRPTR  ps_Pipe;
        BPTR    ps_ReadPipe;
        BPTR    ps_WritePipe;
        LONG    ps_ReturnCode;
        char    ps_Buffer[10];
};


/************************************* declarations *******/
void PipeTask(void);

/************************************* global variables ***/
static const char rcssid[] = "$Id: POpen.c,v 2.5 1994/05/24 21:12:09 simons Exp simons $";


/************************************* program ************/

/****** netsupport.library/POpen ******************************************
*
*    NAME
*       POpen -- starts a command and redirects its input stream to a
*               pipe, whose file handle is returned.
*
*    SYNOPSIS
*       fh = POpen(command, mode);
*       D0         A0       D0
*
*       BPTR    fh;
*       STRPTR  command;
*       LONG    mode;
*
*    FUNCTION
*       This is an equivalent of the UNIX popen() routine, designed for
*       AmigaOS. POpen() is usually used to start an external program
*       that expects data from the input stream or provides data via the
*       output stream. The command is started asynchrony, of course.
*
*    INPUTS
*       command = String with the command to be started. The command
*               should not contain any i/o redirectors like "<file",
*               although it will work. Additionally, you should not
*               start a command via POpen("run ...", MODE_PIPETO) either.
*
*       mode = This is the mode the pipe should be opened in. MODE_PIPETO
*               will start the command to read from the pipe and
*               MODE_PIPEFROM will start the command with its output re-
*               directed to the returned pipe-handle.
*
*    RESULT
*       The routine returns the file handle of the opened pipe, which you
*       can use for either reading or writing, depending on the mode you
*       opened the pipe with. If an error occurs, NULL will be returned.
*
*    EXAMPLE
*       The following code will send some dummy letter to the postmaster
*       of your UUCP/inet site:
*
*               BPTR fh;
*               STRPTR sendmail = GetConfig(NULL, SENDMAIL, NULL, SENDMAIL);
*
*               if (fh = POpen(sendmail, MODE_PIPETO)) {
*                   FPrintf(fh, "To: postmaster\nSubject: Bug report\n\n");
*                   FPuts(fh, "Sorry to bother you, but error #1 occurred\n);
*                   PClose(fh);
*               }
*
*
*    SEE ALSO
*       PClose()
*
*
***************************************************************************
*
*/

BPTR __asm POpen(register __a0 STRPTR command, register __d0 LONG mode)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct PipeStruct *pipestruct;
        struct MsgPort *mp;
        struct Process *pipetask, *proc;
        BPTR pipe_in, pipe_out, pipefh = NULL;
        UBYTE pipename[32];
        ULONG stack;

        if (mode != MODE_PIPETO && mode != MODE_PIPEFROM)       /* check mode! */
                return NULL;


        /*
         * Build the name for the pipe and open it for the
         * pipe process.
         */

        StringCopy(StringCopy(StringCopy(pipename, "PIPE:"), TempName(NULL)), "//0");

        if (pipe_in = Open(pipename, MODE_NEWFILE)) {
                if (pipe_out = Open(pipename, MODE_OLDFILE)) {
                        if (mode == MODE_PIPETO)
                                pipefh = pipe_in;
                        else if (mode == MODE_PIPEFROM)
                                pipefh = pipe_out;
                }
                else
                        Close(pipe_in);
        }
        else
                return NULL;


        /*
         * Determine our stacksize and save it for the pipe process.
         */

        proc = (struct Process *)FindTask(NULL);
        if (proc->pr_Task.tc_Node.ln_Type == NT_PROCESS && proc->pr_CLI)
                stack = ( ((struct CommandLineInterface *)BADDR(proc->pr_CLI))->cli_DefaultStack) << 2;
        else
                stack = proc->pr_StackSize;


        /*
         * Create the message port and start the other process with
         * the correct parameters.
         */

        if (mp = CreateMsgPort()) {
                if (pipestruct = NSPAllocMemPooled(sizeof(struct PipeStruct), MEMF_ANY|MEMF_CLEAR)) {
                        RawDoFmt("%08.lx", &pipefh, StuffChar, &(pipestruct->ps_Buffer[0]));
                        mp->mp_Node.ln_Name = &(pipestruct->ps_Buffer[0]);
                        pipestruct->ps_PipeMsg.mn_Length = sizeof(struct PipeStruct);
                        pipestruct->ps_PipeMsg.mn_ReplyPort = mp;
                        pipestruct->ps_Pipe = command;
                        if (mode == MODE_PIPETO) {
                                pipestruct->ps_ReadPipe = pipe_out;
                                pipestruct->ps_WritePipe = NULL;
                        }
                        else if (mode == MODE_PIPEFROM) {
                                pipestruct->ps_ReadPipe = NULL;
                                pipestruct->ps_WritePipe = pipe_in;
                        }


                        if (pipetask = CreateNewProcTags(NP_Entry, PipeTask, NP_Name, pipename,
                                                         NP_Cli, TRUE, NP_StackSize, stack,
                                                         TAG_DONE)) {
                                PutMsg(&(pipetask->pr_MsgPort), &(pipestruct->ps_PipeMsg));

                                ObtainSemaphore(NetSupportBase->nsp_MsgPortListSemaphore);
                                AddHead((struct List *) &(NetSupportBase->nsp_MsgPortList), &(mp->mp_Node));
                                ReleaseSemaphore(NetSupportBase->nsp_MsgPortListSemaphore);

                                return pipefh;
                        }
                        NSPFreeMemPooled(pipestruct);
                }
                DeleteMsgPort(mp);
        }

        Close(pipe_in);
        Close(pipe_out);

        return NULL;
}


void PipeTask(void)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct PipeStruct *ps;

        WaitPort(&(((struct Process *) FindTask(NULL))->pr_MsgPort));
        ps = (struct PipeStruct *) GetMsg(&(((struct Process *) FindTask(NULL))->pr_MsgPort));
        ps->ps_ReturnCode = SystemTags(ps->ps_Pipe,
                                        SYS_Input, ps->ps_ReadPipe,
                                        SYS_Output, ps->ps_WritePipe,
                                        NP_ConsoleTask, NULL,
                                        TAG_DONE);
        ReplyMsg(&(ps->ps_PipeMsg));
}


/****** netsupport.library/PClose *****************************************
*
*    NAME
*       PClose -- closes a pipe previously opened with POpen()
*
*    SYNOPSIS
*       returncode = PClose(fh);
*       D0                  d0
*
*       LONG    returncode;
*       BPTR    fh;
*
*    FUNCTION
*       This routine closes a pipe previously created with POpen() and
*       returns the returncode of the executed command.
*
*    INPUTS
*       fh = filehandle returned by POpen()
*
*    RESULT
*       Code returned by the executed command.
*
*    BUGS
*       Until NetSupport.Library version 1.27, PClose() did not catch the
*       returncode of the executed program. This has been fixed.
*
*    SEE ALSO
*       POpen()
*
***************************************************************************
*
*/

LONG __asm PClose(register __d0 BPTR fh)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct PipeStruct *pipestruct;
        struct MsgPort *mp;
        char buffer[10];
        BPTR tmp_fh = fh;
        LONG rc;

        RawDoFmt("%08.lx", &tmp_fh, StuffChar, buffer);

        ObtainSemaphore(NetSupportBase->nsp_MsgPortListSemaphore);
        mp = (struct MsgPort *) FindName((struct List *) &(NetSupportBase->nsp_MsgPortList), buffer);
        if (mp)
                Remove(&(mp->mp_Node));
        ReleaseSemaphore(NetSupportBase->nsp_MsgPortListSemaphore);

        if (!mp)
                return -1L;

        Close(fh);
        WaitPort(mp);
        pipestruct = (struct PipeStruct *) GetMsg(mp);

        if (pipestruct->ps_ReadPipe)
                Close(pipestruct->ps_ReadPipe);
        if (pipestruct->ps_WritePipe)
                Close(pipestruct->ps_WritePipe);
        rc = pipestruct->ps_ReturnCode;

        NSPFreeMemPooled(pipestruct);
        DeleteMsgPort(mp);

        return rc;
}

