/*
 *      $Filename: TmpFiles.c $
 *      $Revision: 1.11 $
 *      $Date: 1994/06/29 15:34:06 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: TmpFiles.c,v 1.11 1994/06/29 15:34:06 simons Exp simons $
 *
 */


/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/utility.h>

#include "NetSupportLibrary.h"
#include "MemoryHandling.h"
#include "SupportRoutines.h"
#include "ConfigFiles.h"
#include "LockFiles.h"

/************************************* Defines ************/
#define SEQFILE "SeqFile"

/************************************* global Variables ***/
static const char __RCSId[] = "$Id: TmpFiles.c,v 1.11 1994/06/29 15:34:06 simons Exp simons $";


/**************************************************************************
 *                                                                        *
 * SEKTION: Public library functions                                      *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/TempName ***************************************
*
*   NAME
*       TempName -- build an unique temporary filename
*
*   SYNOPSIS
*       tmpname = TempName(buffer);
*       D0                 A0
*
*       STRPTR tmpname;
*       STRPTR buffer;
*
*   FUNCTION
*       TempName() builds an unique filename you can use for your
*       temporary files.
*
*   INPUTS
*       buffer - Pointer to an buffer to hold the filename. The
*                       generated filename will not succeed 12
*                       characters. If NULL is specified, TempName()
*                       will use an internal buffer and return a pointer
*                       to this one. Please take note, that the internal
*                       buffer will be overwritten the next time you
*                       call TempName(), so copying its contents might be
*                       a good idea.
*
*   RESULT
*       The returned result is a pointer to a buffer holding the name of
*       the unique filename. The filename will look like this:
*       "tmpXXXXXXXX", where the 'X-part' will be a hexadecimal number
*       like "tmp0000a214", for example.
*
*       If an error occurs (out of memory), NULL will be returned. If
*       you provide the buffer, the routine CANNOT fail!
*
*   NOTES
*       These filenames are meant for temporary files and you should use
*       them only for temporary ones. The number part (which make the
*       name unique) will be flushed to zero, every time the library is
*       removed from memory. If you want to generate unique filenames
*       which won't be overwritten after your program terminates, use
*       GetSeq() and similar routines to build the filename.
*
*       If you want the filename to contain the "T:" path for temporary
*       files, call TempNameT(), which will result in "T:tmpXXXXXXXX".
*
*   EXAMPLE
*       Though it doesn't really matter, you might want to place the
*       temporary files in certain directories or let them reflect the
*       program, they belong to. This can be archives as follows:
*
*               char tmpname[32];
*
*               strcpy(tmpname, "SYS:foobar_");
*               strcat(tmpname, TempName(NULL));
*
*       This code will generate a filename like "SYS:foobar_tmp00000001".
*
*   SEE ALSO
*       TempNameT(), GetSeq()
*
***************************************************************************
*
*/

STRPTR __asm TempName(register __a0 STRPTR buffer)
{
        struct StaticBuffer *staticbuf;
        ULONG num;

        if (!buffer) {
                if (!(staticbuf = GetStaticBuffer()))
                        return 0L;
                if (!staticbuf->nspsb_TempNameBuffer) {
                        if (!(staticbuf->nspsb_TempNameBuffer = NSPAllocMemPooled(32, 0L)))
                                return 0L;
                }
                buffer = staticbuf->nspsb_TempNameBuffer;
        }

        Forbid();
        num = NetSupportBase->nsp_UniqueNumber++;
        Permit();

        RawDoFmt("tmp%08.lx", &num, StuffChar, buffer);

        return buffer;
}


/****** netsupport.library/TempNameT **************************************
*
*   NAME
*       TempNameT -- build an unique temporary filename including path
*
*   SYNOPSIS
*       tmpname = TempNameT(buffer);
*       D0                 A0
*
*       STRPTR tmpname;
*       STRPTR buffer;
*
*   FUNCTION
*       TempNameT() builds an unique path and filename you can use for
*       your temporary files.
*
*   INPUTS
*       buffer - Pointer to an buffer to hold the filename. The
*                       generated filename will not succeed 14
*                       characters. If NULL is specified, TempNameT()
*                       will use an internal buffer and return a pointer
*                       to this one. Please take note, that the internal
*                       buffer will be overwritten the next time you
*                       call TempNameT(), so copying its contents might
*                       be a good idea.
*
*   RESULT
*       The returned result is a pointer to a buffer holding the name of
*       the unique filename. The filename will look like this:
*       "T:tmpXXXXXXXX", where the 'X-part' will be a hexadecimal number
*       like "T:tmp0000a214", for example.
*
*       If an error occurs (out of memory), NULL will be returned. If
*       you provide the buffer, the routine CANNOT fail!
*
*   NOTES
*       These filenames are meant for temporary files and you should use
*       them only for temporary ones. The number part (which make the
*       name unique) will be flushed to zero, every time the library is
*       removed from memory. If you want to generate unique filenames
*       which won't be overwritten after your program terminates, use
*       GetSeq() and similar routines to build the filename.
*
*   EXAMPLE
*       Though it doesn't really matter, you might the filename to
*       reflect the program, it belongs to:
*
*               char tmpname[32];
*
*               strcpy(tmpname, TempNameT(NULL));
*               strcat(tmpname, ".foobar");
*
*       This code will generate a filename like "T:tmp00000001.foobar".
*
*   SEE ALSO
*       TempName(), GetSeq()
*
***************************************************************************
*
*/

STRPTR __asm TempNameT(register __a0 STRPTR buffer)
{
        char tmp_buffer[32], *result_buffer;

        StringCopy(tmp_buffer, "T:");
        if (!(result_buffer = TempName(buffer)))
                return NULL;
        StringCat(tmp_buffer, result_buffer);
        StringCopy(result_buffer, tmp_buffer);
        return result_buffer;
}



/****** netsupport.library/GetSeq *****************************************
*
*   NAME
*       GetSeq -- get a sequential, unique number
*
*   SYNOPSIS
*       number = GetSeq(bump)
*       D0              D0
*
*       ULONG   number;
*       ULONG   bump;
*
*   FUNCTION
*       GetSeq() returns a number which is guaranteed to be unique.
*       Despite the internal numbers used by TempName(), this number is
*       still unique when the library was closed and flushed or the machine
*       has been switched off.
*
*       Obviously, this requires the number to be saved on disk.
*
*   INPUTS
*       bump - The returned number is sequential. >bump< is the value that
*                       is added to the current value. Using bump, you
*                       can obtain (allocate) several numbers with one
*                       call--reducing the number of necessary disk
*                       accesses.
*
*   RESULT
*       GetSeq() returns the current value of the counter and writes the
*       increased counter back to disk.
*
*   NOTE
*       GetSeq(0L) returns the current value, but does NOT increase the
*       counter!
*
*       The file that holds the current seq-counter can be configured in
*       the library configfile. Please do never change this value manually
*       unless you know what you do.
*
*   EXAMPLE
*       The current counter is, say, 500. Now you call:
*
*               num = GetSeq(4);
*
*       num will have the value of 501. Additionally, you have the right
*       to use the numbers 502, 503 and 504, because you specified a bump
*       of 4.
*
*   SEE ALSO
*
***************************************************************************
*
*/

ULONG __asm GetSeq(register __d0 bump)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        BPTR fh;
        ULONG seq = 0L, new_seq;
        STRPTR SEQ = GetConfig(GetNSPConfigName(), SEQFILE, NULL, "UULib:NSPLib_Seq");

        LockFile(SEQ);
        do {
                if (fh = Open(SEQ, MODE_OLDFILE)) {
                        Read(fh, &seq, sizeof(ULONG));
                        Close(fh);
                }
                else
                        if (IoErr() == ERROR_OBJECT_NOT_FOUND)
                                break;
                        else
                                Delay(3*50);
        } while (!fh);
        new_seq = seq + bump;
        do {
                if (fh = Open(SEQ, MODE_NEWFILE)) {
                        Write(fh, &new_seq, sizeof(ULONG));
                        Close(fh);
                }
                else
                        Delay(3*50);
        } while (!fh);
        UnLockFile(SEQ);
        return seq;
}


/*************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/

